"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from apideck_unify import models, utils
from apideck_unify._hooks import SDKHooks
from apideck_unify.models import internal
from apideck_unify.types import OptionalNullable, UNSET
import httpx
import importlib
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from apideck_unify.accounting import Accounting
    from apideck_unify.ats import Ats
    from apideck_unify.connector_sdk import ConnectorSDK
    from apideck_unify.crm import Crm
    from apideck_unify.ecommerce import Ecommerce
    from apideck_unify.filestorage import FileStorage
    from apideck_unify.hris import Hris
    from apideck_unify.issuetracking import IssueTracking
    from apideck_unify.sms import Sms
    from apideck_unify.vault import Vault
    from apideck_unify.webhook_sdk import WebhookSDK


class Apideck(BaseSDK):
    r"""Apideck: The Apideck OpenAPI Spec: SDK Optimized
    https://developers.apideck.com - Apideck Developer Docs
    """

    accounting: "Accounting"
    ats: "Ats"
    crm: "Crm"
    ecommerce: "Ecommerce"
    file_storage: "FileStorage"
    hris: "Hris"
    sms: "Sms"
    issue_tracking: "IssueTracking"
    connector: "ConnectorSDK"
    vault: "Vault"
    webhook: "WebhookSDK"
    _sub_sdk_map = {
        "accounting": ("apideck_unify.accounting", "Accounting"),
        "ats": ("apideck_unify.ats", "Ats"),
        "crm": ("apideck_unify.crm", "Crm"),
        "ecommerce": ("apideck_unify.ecommerce", "Ecommerce"),
        "file_storage": ("apideck_unify.filestorage", "FileStorage"),
        "hris": ("apideck_unify.hris", "Hris"),
        "sms": ("apideck_unify.sms", "Sms"),
        "issue_tracking": ("apideck_unify.issuetracking", "IssueTracking"),
        "connector": ("apideck_unify.connector_sdk", "ConnectorSDK"),
        "vault": ("apideck_unify.vault", "Vault"),
        "webhook": ("apideck_unify.webhook_sdk", "WebhookSDK"),
    }

    def __init__(
        self,
        api_key: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key: The api_key required for authentication
        :param consumer_id: Configures the consumer_id parameter for all supported operations
        :param app_id: Configures the app_id parameter for all supported operations
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(api_key=api_key())
        else:
            security = models.Security(api_key=api_key)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        _globals = internal.Globals(
            consumer_id=utils.get_global_from_env(
                consumer_id, "APIDECK_CONSUMER_ID", str
            ),
            app_id=utils.get_global_from_env(app_id, "APIDECK_APP_ID", str),
        )

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                globals=_globals,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = importlib.import_module(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .unifiedapiid import UnifiedAPIID
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class WebhookEventLogServiceTypedDict(TypedDict):
    r"""Apideck service provider associated with event."""

    id: str
    r"""Apideck service provider id."""
    name: str
    r"""Apideck service provider name."""


class WebhookEventLogService(BaseModel):
    r"""Apideck service provider associated with event."""

    id: str
    r"""Apideck service provider id."""

    name: str
    r"""Apideck service provider name."""


class AttemptsTypedDict(TypedDict):
    timestamp: NotRequired[str]
    r"""ISO Date and time when the request was made."""
    execution_attempt: NotRequired[float]
    r"""Number of attempts webhook endpoint was called before a success was returned or eventually failed"""
    status_code: NotRequired[int]
    r"""HTTP Status code that was returned."""
    success: NotRequired[bool]
    r"""Whether or not the request was successful."""


class Attempts(BaseModel):
    timestamp: Optional[str] = None
    r"""ISO Date and time when the request was made."""

    execution_attempt: Optional[float] = None
    r"""Number of attempts webhook endpoint was called before a success was returned or eventually failed"""

    status_code: Optional[int] = None
    r"""HTTP Status code that was returned."""

    success: Optional[bool] = None
    r"""Whether or not the request was successful."""


class WebhookEventLogTypedDict(TypedDict):
    id: NotRequired[str]
    status_code: NotRequired[int]
    r"""HTTP Status code that was returned."""
    success: NotRequired[bool]
    r"""Whether or not the request was successful."""
    application_id: NotRequired[str]
    r"""ID of your Apideck Application"""
    consumer_id: NotRequired[str]
    r"""Unique consumer identifier. You can freely choose a consumer ID yourself. Most of the time, this is an ID of your internal data model that represents a user or account in your system (for example account:12345). If the consumer doesn't exist yet, Vault will upsert a consumer based on your ID."""
    unified_api: NotRequired[UnifiedAPIID]
    r"""Name of Apideck Unified API"""
    service: NotRequired[WebhookEventLogServiceTypedDict]
    r"""Apideck service provider associated with event."""
    endpoint: NotRequired[str]
    r"""The URL of the webhook endpoint."""
    event_type: NotRequired[str]
    r"""Name of source event that webhook is subscribed to."""
    execution_attempt: NotRequired[float]
    r"""Number of attempts webhook endpoint was called before a success was returned or eventually failed"""
    http_method: NotRequired[str]
    r"""HTTP Method of request to endpoint."""
    timestamp: NotRequired[str]
    r"""ISO Date and time when the request was made."""
    entity_type: NotRequired[str]
    r"""Name of the Entity described by the attributes delivered within payload"""
    request_body: NotRequired[str]
    r"""The JSON stringified payload that was delivered to the webhook endpoint."""
    response_body: NotRequired[str]
    r"""The JSON stringified response that was returned from the webhook endpoint."""
    retry_scheduled: NotRequired[bool]
    r"""If the request has not hit the max retry limit and will be retried."""
    attempts: NotRequired[List[AttemptsTypedDict]]
    r"""record of each attempt to call webhook endpoint"""


class WebhookEventLog(BaseModel):
    id: Optional[str] = None

    status_code: Optional[int] = None
    r"""HTTP Status code that was returned."""

    success: Optional[bool] = None
    r"""Whether or not the request was successful."""

    application_id: Optional[str] = None
    r"""ID of your Apideck Application"""

    consumer_id: Optional[str] = None
    r"""Unique consumer identifier. You can freely choose a consumer ID yourself. Most of the time, this is an ID of your internal data model that represents a user or account in your system (for example account:12345). If the consumer doesn't exist yet, Vault will upsert a consumer based on your ID."""

    unified_api: Optional[UnifiedAPIID] = None
    r"""Name of Apideck Unified API"""

    service: Optional[WebhookEventLogService] = None
    r"""Apideck service provider associated with event."""

    endpoint: Optional[str] = None
    r"""The URL of the webhook endpoint."""

    event_type: Optional[str] = None
    r"""Name of source event that webhook is subscribed to."""

    execution_attempt: Optional[float] = None
    r"""Number of attempts webhook endpoint was called before a success was returned or eventually failed"""

    http_method: Optional[str] = None
    r"""HTTP Method of request to endpoint."""

    timestamp: Optional[str] = None
    r"""ISO Date and time when the request was made."""

    entity_type: Optional[str] = None
    r"""Name of the Entity described by the attributes delivered within payload"""

    request_body: Optional[str] = None
    r"""The JSON stringified payload that was delivered to the webhook endpoint."""

    response_body: Optional[str] = None
    r"""The JSON stringified response that was returned from the webhook endpoint."""

    retry_scheduled: Optional[bool] = None
    r"""If the request has not hit the max retry limit and will be retried."""

    attempts: Optional[List[Attempts]] = None
    r"""record of each attempt to call webhook endpoint"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.collections import Collections
from apideck_unify.collectiontags import CollectionTags
from apideck_unify.collectionticketcomments import CollectionTicketComments
from apideck_unify.collectiontickets import CollectionTickets
from apideck_unify.collectionusers import CollectionUsers


class IssueTracking(BaseSDK):
    collections: Collections
    collection_tickets: CollectionTickets
    collection_ticket_comments: CollectionTicketComments
    collection_users: CollectionUsers
    collection_tags: CollectionTags

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.collections = Collections(self.sdk_configuration)
        self.collection_tickets = CollectionTickets(self.sdk_configuration)
        self.collection_ticket_comments = CollectionTicketComments(
            self.sdk_configuration
        )
        self.collection_users = CollectionUsers(self.sdk_configuration)
        self.collection_tags = CollectionTags(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RebillStatus(str, Enum):
    r"""Status of the rebilling process for this line item."""

    PENDING = "pending"
    BILLED = "billed"
    VOIDED = "voided"


class RebillingTypedDict(TypedDict):
    r"""Rebilling metadata for this line item."""

    rebillable: NotRequired[bool]
    r"""Whether this line item is eligible for rebilling."""
    rebill_status: NotRequired[Nullable[RebillStatus]]
    r"""Status of the rebilling process for this line item."""
    linked_transaction_id: NotRequired[Nullable[str]]
    r"""The ID of the transaction this line item was rebilled to."""
    linked_transaction_line_id: NotRequired[Nullable[str]]
    r"""The ID of the line item in the rebilled transaction."""


class Rebilling(BaseModel):
    r"""Rebilling metadata for this line item."""

    rebillable: Optional[bool] = None
    r"""Whether this line item is eligible for rebilling."""

    rebill_status: OptionalNullable[RebillStatus] = UNSET
    r"""Status of the rebilling process for this line item."""

    linked_transaction_id: OptionalNullable[str] = UNSET
    r"""The ID of the transaction this line item was rebilled to."""

    linked_transaction_line_id: OptionalNullable[str] = UNSET
    r"""The ID of the line item in the rebilled transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "rebillable",
            "rebill_status",
            "linked_transaction_id",
            "linked_transaction_line_id",
        ]
        nullable_fields = [
            "rebill_status",
            "linked_transaction_id",
            "linked_transaction_line_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProfitAndLossFilterTypedDict(TypedDict):
    customer_id: NotRequired[str]
    r"""Filter by customer id"""
    start_date: NotRequired[str]
    r"""Filter by start date. If start date is given, end date is required."""
    end_date: NotRequired[str]
    r"""Filter by end date. If end date is given, start date is required."""
    location_id: NotRequired[str]
    r"""Filter by location id"""


class ProfitAndLossFilter(BaseModel):
    customer_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by customer id"""

    start_date: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by start date. If start date is given, end date is required."""

    end_date: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by end date. If end date is given, start date is required."""

    location_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Filter by location id"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivitiesFilterTypedDict(TypedDict):
    company_id: NotRequired[str]
    r"""Company ID to filter on"""
    owner_id: NotRequired[str]
    r"""Owner ID to filter on"""
    contact_id: NotRequired[str]
    r"""Primary contact ID to filter on"""
    updated_since: NotRequired[datetime]
    type: NotRequired[str]
    r"""Type to filter on"""


class ActivitiesFilter(BaseModel):
    company_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Company ID to filter on"""

    owner_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Owner ID to filter on"""

    contact_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Primary contact ID to filter on"""

    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None

    type: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Type to filter on"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CustomObjectSchemaType(str, Enum):
    STRING = "string"
    NUMBER = "number"
    INTEGER = "integer"
    BOOLEAN = "boolean"
    DATE = "date"
    DATETIME = "datetime"
    CURRENCY = "currency"
    EMAIL = "email"
    PHONE = "phone"
    REFERENCE = "reference"
    SELECT = "select"
    MULTISELECT = "multiselect"


class CustomObjectSchemaOptionsTypedDict(TypedDict):
    value: NotRequired[str]
    label: NotRequired[str]


class CustomObjectSchemaOptions(BaseModel):
    value: Optional[str] = None

    label: Optional[str] = None


class FieldsTypedDict(TypedDict):
    id: NotRequired[str]
    name: NotRequired[str]
    description: NotRequired[Nullable[str]]
    type: NotRequired[CustomObjectSchemaType]
    required: NotRequired[bool]
    options: NotRequired[Nullable[List[CustomObjectSchemaOptionsTypedDict]]]
    r"""Options for select and multiselect types"""
    default_value: NotRequired[Nullable[str]]
    r"""Default value for the field"""


class Fields(BaseModel):
    id: Optional[str] = None

    name: Optional[str] = None

    description: OptionalNullable[str] = UNSET

    type: Optional[CustomObjectSchemaType] = None

    required: Optional[bool] = None

    options: OptionalNullable[List[CustomObjectSchemaOptions]] = UNSET
    r"""Options for select and multiselect types"""

    default_value: OptionalNullable[str] = UNSET
    r"""Default value for the field"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "description",
            "type",
            "required",
            "options",
            "default_value",
        ]
        nullable_fields = ["description", "options", "default_value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomObjectSchemaTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The unique identifier of the custom object schema"""
    name: NotRequired[Nullable[str]]
    r"""The name of the custom object schema"""
    description: NotRequired[Nullable[str]]
    r"""The description of the custom object schema"""
    fields: NotRequired[List[FieldsTypedDict]]
    r"""The fields defined in the schema"""
    visible: NotRequired[Nullable[bool]]
    r"""Whether the custom object schema is visible in the UI"""
    active: NotRequired[Nullable[bool]]
    r"""Whether the custom object schema is active"""
    created_by: NotRequired[Nullable[str]]
    r"""The ID of the user who created the custom object schema"""
    updated_by: NotRequired[Nullable[str]]
    r"""The ID of the user who last updated the custom object schema"""
    updated_at: NotRequired[Nullable[str]]
    r"""The timestamp when the custom object schema was last updated"""
    created_at: NotRequired[Nullable[str]]
    r"""The timestamp when the custom object schema was created"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CustomObjectSchema(BaseModel):
    id: Optional[str] = None
    r"""The unique identifier of the custom object schema"""

    name: OptionalNullable[str] = UNSET
    r"""The name of the custom object schema"""

    description: OptionalNullable[str] = UNSET
    r"""The description of the custom object schema"""

    fields: Optional[List[Fields]] = None
    r"""The fields defined in the schema"""

    visible: OptionalNullable[bool] = UNSET
    r"""Whether the custom object schema is visible in the UI"""

    active: OptionalNullable[bool] = UNSET
    r"""Whether the custom object schema is active"""

    created_by: OptionalNullable[str] = UNSET
    r"""The ID of the user who created the custom object schema"""

    updated_by: OptionalNullable[str] = UNSET
    r"""The ID of the user who last updated the custom object schema"""

    updated_at: OptionalNullable[str] = UNSET
    r"""The timestamp when the custom object schema was last updated"""

    created_at: OptionalNullable[str] = UNSET
    r"""The timestamp when the custom object schema was created"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "description",
            "fields",
            "visible",
            "active",
            "created_by",
            "updated_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "name",
            "description",
            "visible",
            "active",
            "created_by",
            "updated_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomObjectSchemaInputTypedDict(TypedDict):
    name: NotRequired[Nullable[str]]
    r"""The name of the custom object schema"""
    description: NotRequired[Nullable[str]]
    r"""The description of the custom object schema"""
    fields: NotRequired[List[FieldsTypedDict]]
    r"""The fields defined in the schema"""
    visible: NotRequired[Nullable[bool]]
    r"""Whether the custom object schema is visible in the UI"""
    active: NotRequired[Nullable[bool]]
    r"""Whether the custom object schema is active"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CustomObjectSchemaInput(BaseModel):
    name: OptionalNullable[str] = UNSET
    r"""The name of the custom object schema"""

    description: OptionalNullable[str] = UNSET
    r"""The description of the custom object schema"""

    fields: Optional[List[Fields]] = None
    r"""The fields defined in the schema"""

    visible: OptionalNullable[bool] = UNSET
    r"""Whether the custom object schema is visible in the UI"""

    active: OptionalNullable[bool] = UNSET
    r"""Whether the custom object schema is active"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "description",
            "fields",
            "visible",
            "active",
            "pass_through",
        ]
        nullable_fields = ["name", "description", "visible", "active"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SortDirection(str, Enum):
    r"""The direction in which to sort the results"""

    ASC = "asc"
    DESC = "desc"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PaginationCoverageMode(str, Enum):
    r"""How pagination is implemented on this connector. Native mode means Apideck is using the pagination parameters of the connector. With virtual pagination, the connector does not support pagination, but Apideck emulates it."""

    NATIVE = "native"
    VIRTUAL = "virtual"


class PaginationCoverageTypedDict(TypedDict):
    mode: NotRequired[PaginationCoverageMode]
    r"""How pagination is implemented on this connector. Native mode means Apideck is using the pagination parameters of the connector. With virtual pagination, the connector does not support pagination, but Apideck emulates it."""
    paging_support: NotRequired[bool]
    r"""Indicates whether the connector supports paging through results using the cursor parameter."""
    limit_support: NotRequired[bool]
    r"""Indicates whether the connector supports changing the page size by using the limit parameter."""


class PaginationCoverage(BaseModel):
    mode: Optional[PaginationCoverageMode] = None
    r"""How pagination is implemented on this connector. Native mode means Apideck is using the pagination parameters of the connector. With virtual pagination, the connector does not support pagination, but Apideck emulates it."""

    paging_support: Optional[bool] = None
    r"""Indicates whether the connector supports paging through results using the cursor parameter."""

    limit_support: Optional[bool] = None
    r"""Indicates whether the connector supports changing the page size by using the limit parameter."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class CollectionTicketCommentTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    body: NotRequired[Nullable[str]]
    r"""Body of the comment"""
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CollectionTicketComment(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    body: OptionalNullable[str] = UNSET
    r"""Body of the comment"""

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "body",
            "custom_mappings",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "body",
            "custom_mappings",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EcommerceProductsFilterTypedDict(TypedDict):
    name: NotRequired[str]
    r"""Product name to filter on"""
    updated_since: NotRequired[str]
    r"""Minimum date the product was last modified"""
    created_since: NotRequired[str]
    r"""Minimum date the product was created"""


class EcommerceProductsFilter(BaseModel):
    name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Product name to filter on"""

    updated_since: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Minimum date the product was last modified"""

    created_since: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Minimum date the product was created"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class OpportunitiesFilterTypedDict(TypedDict):
    title: NotRequired[str]
    r"""Title of the opportunity to filter on"""
    status: NotRequired[str]
    r"""Status to filter on"""
    monetary_amount: NotRequired[float]
    r"""Monetary amount to filter on"""
    win_probability: NotRequired[float]
    r"""Win probability to filter on"""
    company_id: NotRequired[str]
    r"""Company ID to filter on"""
    owner_id: NotRequired[str]
    r"""Owner ID to filter on"""
    primary_contact_id: NotRequired[str]
    r"""Primary contact ID to filter on"""


class OpportunitiesFilter(BaseModel):
    title: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Title of the opportunity to filter on"""

    status: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Status to filter on"""

    monetary_amount: Annotated[Optional[float], FieldMetadata(query=True)] = None
    r"""Monetary amount to filter on"""

    win_probability: Annotated[Optional[float], FieldMetadata(query=True)] = None
    r"""Win probability to filter on"""

    company_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Company ID to filter on"""

    owner_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Owner ID to filter on"""

    primary_contact_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Primary contact ID to filter on"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import Nullable, OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from apideck_unify.utils.unmarshal_json_response import unmarshal_json_response
from datetime import date
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Employees(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.EmployeesFilter, models.EmployeesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.EmployeesSort, models.EmployeesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.HrisEmployeesAllResponse]:
        r"""List Employees

        Apideck operates as a stateless Unified API, which means that the list endpoint only provides a portion of the employee model. This is due to the fact that most HRIS systems do not readily provide all data in every call. However, you can access the complete employee model through an employee detail call.

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(filter_, Optional[models.EmployeesFilter]),
            sort=utils.get_pydantic_model(sort, Optional[models.EmployeesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/hris/employees",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.HrisEmployeesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesAllResponse(
                get_employees_response=unmarshal_json_response(
                    Optional[models.GetEmployeesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.EmployeesFilter, models.EmployeesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.EmployeesSort, models.EmployeesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.HrisEmployeesAllResponse]:
        r"""List Employees

        Apideck operates as a stateless Unified API, which means that the list endpoint only provides a portion of the employee model. This is due to the fact that most HRIS systems do not readily provide all data in every call. However, you can access the complete employee model through an employee detail call.

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(filter_, Optional[models.EmployeesFilter]),
            sort=utils.get_pydantic_model(sort, Optional[models.EmployeesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/hris/employees",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.HrisEmployeesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesAllResponse(
                get_employees_response=unmarshal_json_response(
                    Optional[models.GetEmployeesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        id: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        middle_name: OptionalNullable[str] = UNSET,
        display_name: OptionalNullable[str] = UNSET,
        preferred_name: OptionalNullable[str] = UNSET,
        initials: OptionalNullable[str] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        marital_status: OptionalNullable[str] = UNSET,
        partner: Optional[
            Union[models.PersonInput, models.PersonInputTypedDict]
        ] = None,
        division: OptionalNullable[str] = UNSET,
        division_id: OptionalNullable[str] = UNSET,
        department: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        department_name: OptionalNullable[str] = UNSET,
        team: OptionalNullable[Union[models.Team, models.TeamTypedDict]] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        employment_start_date: OptionalNullable[str] = UNSET,
        employment_end_date: OptionalNullable[str] = UNSET,
        leaving_reason: OptionalNullable[models.LeavingReason] = UNSET,
        employee_number: OptionalNullable[str] = UNSET,
        employment_status: OptionalNullable[models.EmploymentStatus] = UNSET,
        employment_role: Optional[
            Union[models.EmploymentRole, models.EmploymentRoleTypedDict]
        ] = None,
        ethnicity: OptionalNullable[str] = UNSET,
        manager: Optional[Union[models.Manager, models.ManagerTypedDict]] = None,
        direct_reports: OptionalNullable[List[str]] = UNSET,
        social_security_number: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        deceased_on: OptionalNullable[date] = UNSET,
        country_of_birth: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        gender: OptionalNullable[models.Gender] = UNSET,
        pronouns: OptionalNullable[str] = UNSET,
        preferred_language: OptionalNullable[str] = UNSET,
        languages: Optional[List[Nullable[str]]] = None,
        nationalities: Optional[List[Nullable[str]]] = None,
        photo_url: OptionalNullable[str] = UNSET,
        timezone: OptionalNullable[str] = UNSET,
        source: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        record_url: OptionalNullable[str] = UNSET,
        jobs: OptionalNullable[
            Union[List[models.EmployeeJobInput], List[models.EmployeeJobInputTypedDict]]
        ] = UNSET,
        compensations: OptionalNullable[
            Union[
                List[models.EmployeeCompensationInput],
                List[models.EmployeeCompensationInputTypedDict],
            ]
        ] = UNSET,
        works_remote: OptionalNullable[bool] = UNSET,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        bank_accounts: Optional[
            Union[List[models.BankAccount], List[models.BankAccountTypedDict]]
        ] = None,
        tax_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        dietary_preference: OptionalNullable[str] = UNSET,
        food_allergies: OptionalNullable[List[str]] = UNSET,
        probation_period: Optional[
            Union[models.ProbationPeriod, models.ProbationPeriodTypedDict]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        row_version: OptionalNullable[str] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesAddResponse:
        r"""Create Employee

        Create Employee

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param id: A unique identifier for an object.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param middle_name: Middle name of the person.
        :param display_name: The name used to display the employee, often a combination of their first and last names.
        :param preferred_name: The name the employee prefers to be addressed by, which may be different from their legal name.
        :param initials: The initials of the person, usually derived from their first, middle, and last names.
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param title: The job title of the person.
        :param marital_status: The marital status of the employee.
        :param partner:
        :param division: The division the person is currently in. Usually a collection of departments or teams or regions.
        :param division_id: Unique identifier of the division this employee belongs to.
        :param department: The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field.
        :param department_id: Unique identifier of the department ID this employee belongs to.
        :param department_name: Name of the department this employee belongs to.
        :param team: The team the person is currently in.
        :param company_id: The unique identifier of the company.
        :param company_name: The name of the company.
        :param employment_start_date: A Start Date is the date that the employee started working at the company
        :param employment_end_date: An End Date is the date that the employee ended working at the company
        :param leaving_reason: The reason because the employment ended.
        :param employee_number: An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company.
        :param employment_status: The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status.
        :param employment_role:
        :param ethnicity: The ethnicity of the employee
        :param manager:
        :param direct_reports: Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee.
        :param social_security_number: A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions.
        :param birthday: The date of birth of the person.
        :param deceased_on: The date the person deceased.
        :param country_of_birth: Country code according to ISO 3166-1 alpha-2.
        :param description: A description of the object.
        :param gender: The gender represents the gender identity of a person.
        :param pronouns: The preferred pronouns of the person.
        :param preferred_language: language code according to ISO 639-1. For the United States - EN
        :param languages:
        :param nationalities:
        :param photo_url: The URL of the photo of a person.
        :param timezone: The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London.
        :param source: When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from.
        :param source_id: Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS).
        :param record_url:
        :param jobs:
        :param compensations:
        :param works_remote: Indicates if the employee works from a remote location.
        :param addresses:
        :param phone_numbers:
        :param emails:
        :param custom_fields:
        :param social_links:
        :param bank_accounts:
        :param tax_code:
        :param tax_id:
        :param dietary_preference: Indicate the employee's dietary preference.
        :param food_allergies: Indicate the employee's food allergies.
        :param probation_period:
        :param tags:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param deleted: Flag to indicate if the object is deleted.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            employee=models.EmployeeInput(
                id=id,
                first_name=first_name,
                last_name=last_name,
                middle_name=middle_name,
                display_name=display_name,
                preferred_name=preferred_name,
                initials=initials,
                salutation=salutation,
                title=title,
                marital_status=marital_status,
                partner=utils.get_pydantic_model(partner, Optional[models.PersonInput]),
                division=division,
                division_id=division_id,
                department=department,
                department_id=department_id,
                department_name=department_name,
                team=utils.get_pydantic_model(team, OptionalNullable[models.Team]),
                company_id=company_id,
                company_name=company_name,
                employment_start_date=employment_start_date,
                employment_end_date=employment_end_date,
                leaving_reason=leaving_reason,
                employee_number=employee_number,
                employment_status=employment_status,
                employment_role=utils.get_pydantic_model(
                    employment_role, Optional[models.EmploymentRole]
                ),
                ethnicity=ethnicity,
                manager=utils.get_pydantic_model(manager, Optional[models.Manager]),
                direct_reports=direct_reports,
                social_security_number=social_security_number,
                birthday=birthday,
                deceased_on=deceased_on,
                country_of_birth=country_of_birth,
                description=description,
                gender=gender,
                pronouns=pronouns,
                preferred_language=preferred_language,
                languages=languages,
                nationalities=nationalities,
                photo_url=photo_url,
                timezone=timezone,
                source=source,
                source_id=source_id,
                record_url=record_url,
                jobs=utils.get_pydantic_model(
                    jobs, OptionalNullable[List[models.EmployeeJobInput]]
                ),
                compensations=utils.get_pydantic_model(
                    compensations,
                    OptionalNullable[List[models.EmployeeCompensationInput]],
                ),
                works_remote=works_remote,
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount]]
                ),
                tax_code=tax_code,
                tax_id=tax_id,
                dietary_preference=dietary_preference,
                food_allergies=food_allergies,
                probation_period=utils.get_pydantic_model(
                    probation_period, Optional[models.ProbationPeriod]
                ),
                tags=tags,
                row_version=row_version,
                deleted=deleted,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/hris/employees",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.employee, False, False, "json", models.EmployeeInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.HrisEmployeesAddResponse(
                create_employee_response=unmarshal_json_response(
                    Optional[models.CreateEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        id: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        middle_name: OptionalNullable[str] = UNSET,
        display_name: OptionalNullable[str] = UNSET,
        preferred_name: OptionalNullable[str] = UNSET,
        initials: OptionalNullable[str] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        marital_status: OptionalNullable[str] = UNSET,
        partner: Optional[
            Union[models.PersonInput, models.PersonInputTypedDict]
        ] = None,
        division: OptionalNullable[str] = UNSET,
        division_id: OptionalNullable[str] = UNSET,
        department: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        department_name: OptionalNullable[str] = UNSET,
        team: OptionalNullable[Union[models.Team, models.TeamTypedDict]] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        employment_start_date: OptionalNullable[str] = UNSET,
        employment_end_date: OptionalNullable[str] = UNSET,
        leaving_reason: OptionalNullable[models.LeavingReason] = UNSET,
        employee_number: OptionalNullable[str] = UNSET,
        employment_status: OptionalNullable[models.EmploymentStatus] = UNSET,
        employment_role: Optional[
            Union[models.EmploymentRole, models.EmploymentRoleTypedDict]
        ] = None,
        ethnicity: OptionalNullable[str] = UNSET,
        manager: Optional[Union[models.Manager, models.ManagerTypedDict]] = None,
        direct_reports: OptionalNullable[List[str]] = UNSET,
        social_security_number: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        deceased_on: OptionalNullable[date] = UNSET,
        country_of_birth: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        gender: OptionalNullable[models.Gender] = UNSET,
        pronouns: OptionalNullable[str] = UNSET,
        preferred_language: OptionalNullable[str] = UNSET,
        languages: Optional[List[Nullable[str]]] = None,
        nationalities: Optional[List[Nullable[str]]] = None,
        photo_url: OptionalNullable[str] = UNSET,
        timezone: OptionalNullable[str] = UNSET,
        source: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        record_url: OptionalNullable[str] = UNSET,
        jobs: OptionalNullable[
            Union[List[models.EmployeeJobInput], List[models.EmployeeJobInputTypedDict]]
        ] = UNSET,
        compensations: OptionalNullable[
            Union[
                List[models.EmployeeCompensationInput],
                List[models.EmployeeCompensationInputTypedDict],
            ]
        ] = UNSET,
        works_remote: OptionalNullable[bool] = UNSET,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        bank_accounts: Optional[
            Union[List[models.BankAccount], List[models.BankAccountTypedDict]]
        ] = None,
        tax_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        dietary_preference: OptionalNullable[str] = UNSET,
        food_allergies: OptionalNullable[List[str]] = UNSET,
        probation_period: Optional[
            Union[models.ProbationPeriod, models.ProbationPeriodTypedDict]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        row_version: OptionalNullable[str] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesAddResponse:
        r"""Create Employee

        Create Employee

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param id: A unique identifier for an object.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param middle_name: Middle name of the person.
        :param display_name: The name used to display the employee, often a combination of their first and last names.
        :param preferred_name: The name the employee prefers to be addressed by, which may be different from their legal name.
        :param initials: The initials of the person, usually derived from their first, middle, and last names.
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param title: The job title of the person.
        :param marital_status: The marital status of the employee.
        :param partner:
        :param division: The division the person is currently in. Usually a collection of departments or teams or regions.
        :param division_id: Unique identifier of the division this employee belongs to.
        :param department: The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field.
        :param department_id: Unique identifier of the department ID this employee belongs to.
        :param department_name: Name of the department this employee belongs to.
        :param team: The team the person is currently in.
        :param company_id: The unique identifier of the company.
        :param company_name: The name of the company.
        :param employment_start_date: A Start Date is the date that the employee started working at the company
        :param employment_end_date: An End Date is the date that the employee ended working at the company
        :param leaving_reason: The reason because the employment ended.
        :param employee_number: An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company.
        :param employment_status: The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status.
        :param employment_role:
        :param ethnicity: The ethnicity of the employee
        :param manager:
        :param direct_reports: Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee.
        :param social_security_number: A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions.
        :param birthday: The date of birth of the person.
        :param deceased_on: The date the person deceased.
        :param country_of_birth: Country code according to ISO 3166-1 alpha-2.
        :param description: A description of the object.
        :param gender: The gender represents the gender identity of a person.
        :param pronouns: The preferred pronouns of the person.
        :param preferred_language: language code according to ISO 639-1. For the United States - EN
        :param languages:
        :param nationalities:
        :param photo_url: The URL of the photo of a person.
        :param timezone: The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London.
        :param source: When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from.
        :param source_id: Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS).
        :param record_url:
        :param jobs:
        :param compensations:
        :param works_remote: Indicates if the employee works from a remote location.
        :param addresses:
        :param phone_numbers:
        :param emails:
        :param custom_fields:
        :param social_links:
        :param bank_accounts:
        :param tax_code:
        :param tax_id:
        :param dietary_preference: Indicate the employee's dietary preference.
        :param food_allergies: Indicate the employee's food allergies.
        :param probation_period:
        :param tags:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param deleted: Flag to indicate if the object is deleted.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            employee=models.EmployeeInput(
                id=id,
                first_name=first_name,
                last_name=last_name,
                middle_name=middle_name,
                display_name=display_name,
                preferred_name=preferred_name,
                initials=initials,
                salutation=salutation,
                title=title,
                marital_status=marital_status,
                partner=utils.get_pydantic_model(partner, Optional[models.PersonInput]),
                division=division,
                division_id=division_id,
                department=department,
                department_id=department_id,
                department_name=department_name,
                team=utils.get_pydantic_model(team, OptionalNullable[models.Team]),
                company_id=company_id,
                company_name=company_name,
                employment_start_date=employment_start_date,
                employment_end_date=employment_end_date,
                leaving_reason=leaving_reason,
                employee_number=employee_number,
                employment_status=employment_status,
                employment_role=utils.get_pydantic_model(
                    employment_role, Optional[models.EmploymentRole]
                ),
                ethnicity=ethnicity,
                manager=utils.get_pydantic_model(manager, Optional[models.Manager]),
                direct_reports=direct_reports,
                social_security_number=social_security_number,
                birthday=birthday,
                deceased_on=deceased_on,
                country_of_birth=country_of_birth,
                description=description,
                gender=gender,
                pronouns=pronouns,
                preferred_language=preferred_language,
                languages=languages,
                nationalities=nationalities,
                photo_url=photo_url,
                timezone=timezone,
                source=source,
                source_id=source_id,
                record_url=record_url,
                jobs=utils.get_pydantic_model(
                    jobs, OptionalNullable[List[models.EmployeeJobInput]]
                ),
                compensations=utils.get_pydantic_model(
                    compensations,
                    OptionalNullable[List[models.EmployeeCompensationInput]],
                ),
                works_remote=works_remote,
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount]]
                ),
                tax_code=tax_code,
                tax_id=tax_id,
                dietary_preference=dietary_preference,
                food_allergies=food_allergies,
                probation_period=utils.get_pydantic_model(
                    probation_period, Optional[models.ProbationPeriod]
                ),
                tags=tags,
                row_version=row_version,
                deleted=deleted,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/hris/employees",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.employee, False, False, "json", models.EmployeeInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.HrisEmployeesAddResponse(
                create_employee_response=unmarshal_json_response(
                    Optional[models.CreateEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        filter_: Optional[
            Union[models.EmployeesOneFilter, models.EmployeesOneFilterTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesOneResponse:
        r"""Get Employee

        Get Employee

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param filter_: Apply filters
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.EmployeesOneFilter]
            ),
            pass_through=pass_through,
        )

        req = self._build_request(
            method="GET",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesOneResponse(
                get_employee_response=unmarshal_json_response(
                    Optional[models.GetEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        fields: OptionalNullable[str] = UNSET,
        filter_: Optional[
            Union[models.EmployeesOneFilter, models.EmployeesOneFilterTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesOneResponse:
        r"""Get Employee

        Get Employee

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param filter_: Apply filters
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            fields=fields,
            filter_=utils.get_pydantic_model(
                filter_, Optional[models.EmployeesOneFilter]
            ),
            pass_through=pass_through,
        )

        req = self._build_request_async(
            method="GET",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesOneResponse(
                get_employee_response=unmarshal_json_response(
                    Optional[models.GetEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        id_param: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        id: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        middle_name: OptionalNullable[str] = UNSET,
        display_name: OptionalNullable[str] = UNSET,
        preferred_name: OptionalNullable[str] = UNSET,
        initials: OptionalNullable[str] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        marital_status: OptionalNullable[str] = UNSET,
        partner: Optional[
            Union[models.PersonInput, models.PersonInputTypedDict]
        ] = None,
        division: OptionalNullable[str] = UNSET,
        division_id: OptionalNullable[str] = UNSET,
        department: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        department_name: OptionalNullable[str] = UNSET,
        team: OptionalNullable[Union[models.Team, models.TeamTypedDict]] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        employment_start_date: OptionalNullable[str] = UNSET,
        employment_end_date: OptionalNullable[str] = UNSET,
        leaving_reason: OptionalNullable[models.LeavingReason] = UNSET,
        employee_number: OptionalNullable[str] = UNSET,
        employment_status: OptionalNullable[models.EmploymentStatus] = UNSET,
        employment_role: Optional[
            Union[models.EmploymentRole, models.EmploymentRoleTypedDict]
        ] = None,
        ethnicity: OptionalNullable[str] = UNSET,
        manager: Optional[Union[models.Manager, models.ManagerTypedDict]] = None,
        direct_reports: OptionalNullable[List[str]] = UNSET,
        social_security_number: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        deceased_on: OptionalNullable[date] = UNSET,
        country_of_birth: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        gender: OptionalNullable[models.Gender] = UNSET,
        pronouns: OptionalNullable[str] = UNSET,
        preferred_language: OptionalNullable[str] = UNSET,
        languages: Optional[List[Nullable[str]]] = None,
        nationalities: Optional[List[Nullable[str]]] = None,
        photo_url: OptionalNullable[str] = UNSET,
        timezone: OptionalNullable[str] = UNSET,
        source: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        record_url: OptionalNullable[str] = UNSET,
        jobs: OptionalNullable[
            Union[List[models.EmployeeJobInput], List[models.EmployeeJobInputTypedDict]]
        ] = UNSET,
        compensations: OptionalNullable[
            Union[
                List[models.EmployeeCompensationInput],
                List[models.EmployeeCompensationInputTypedDict],
            ]
        ] = UNSET,
        works_remote: OptionalNullable[bool] = UNSET,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        bank_accounts: Optional[
            Union[List[models.BankAccount], List[models.BankAccountTypedDict]]
        ] = None,
        tax_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        dietary_preference: OptionalNullable[str] = UNSET,
        food_allergies: OptionalNullable[List[str]] = UNSET,
        probation_period: Optional[
            Union[models.ProbationPeriod, models.ProbationPeriodTypedDict]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        row_version: OptionalNullable[str] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesUpdateResponse:
        r"""Update Employee

        Update Employee

        :param id_param: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param id: A unique identifier for an object.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param middle_name: Middle name of the person.
        :param display_name: The name used to display the employee, often a combination of their first and last names.
        :param preferred_name: The name the employee prefers to be addressed by, which may be different from their legal name.
        :param initials: The initials of the person, usually derived from their first, middle, and last names.
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param title: The job title of the person.
        :param marital_status: The marital status of the employee.
        :param partner:
        :param division: The division the person is currently in. Usually a collection of departments or teams or regions.
        :param division_id: Unique identifier of the division this employee belongs to.
        :param department: The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field.
        :param department_id: Unique identifier of the department ID this employee belongs to.
        :param department_name: Name of the department this employee belongs to.
        :param team: The team the person is currently in.
        :param company_id: The unique identifier of the company.
        :param company_name: The name of the company.
        :param employment_start_date: A Start Date is the date that the employee started working at the company
        :param employment_end_date: An End Date is the date that the employee ended working at the company
        :param leaving_reason: The reason because the employment ended.
        :param employee_number: An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company.
        :param employment_status: The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status.
        :param employment_role:
        :param ethnicity: The ethnicity of the employee
        :param manager:
        :param direct_reports: Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee.
        :param social_security_number: A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions.
        :param birthday: The date of birth of the person.
        :param deceased_on: The date the person deceased.
        :param country_of_birth: Country code according to ISO 3166-1 alpha-2.
        :param description: A description of the object.
        :param gender: The gender represents the gender identity of a person.
        :param pronouns: The preferred pronouns of the person.
        :param preferred_language: language code according to ISO 639-1. For the United States - EN
        :param languages:
        :param nationalities:
        :param photo_url: The URL of the photo of a person.
        :param timezone: The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London.
        :param source: When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from.
        :param source_id: Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS).
        :param record_url:
        :param jobs:
        :param compensations:
        :param works_remote: Indicates if the employee works from a remote location.
        :param addresses:
        :param phone_numbers:
        :param emails:
        :param custom_fields:
        :param social_links:
        :param bank_accounts:
        :param tax_code:
        :param tax_id:
        :param dietary_preference: Indicate the employee's dietary preference.
        :param food_allergies: Indicate the employee's food allergies.
        :param probation_period:
        :param tags:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param deleted: Flag to indicate if the object is deleted.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesUpdateRequest(
            id_param=id_param,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            employee=models.EmployeeInput(
                id=id,
                first_name=first_name,
                last_name=last_name,
                middle_name=middle_name,
                display_name=display_name,
                preferred_name=preferred_name,
                initials=initials,
                salutation=salutation,
                title=title,
                marital_status=marital_status,
                partner=utils.get_pydantic_model(partner, Optional[models.PersonInput]),
                division=division,
                division_id=division_id,
                department=department,
                department_id=department_id,
                department_name=department_name,
                team=utils.get_pydantic_model(team, OptionalNullable[models.Team]),
                company_id=company_id,
                company_name=company_name,
                employment_start_date=employment_start_date,
                employment_end_date=employment_end_date,
                leaving_reason=leaving_reason,
                employee_number=employee_number,
                employment_status=employment_status,
                employment_role=utils.get_pydantic_model(
                    employment_role, Optional[models.EmploymentRole]
                ),
                ethnicity=ethnicity,
                manager=utils.get_pydantic_model(manager, Optional[models.Manager]),
                direct_reports=direct_reports,
                social_security_number=social_security_number,
                birthday=birthday,
                deceased_on=deceased_on,
                country_of_birth=country_of_birth,
                description=description,
                gender=gender,
                pronouns=pronouns,
                preferred_language=preferred_language,
                languages=languages,
                nationalities=nationalities,
                photo_url=photo_url,
                timezone=timezone,
                source=source,
                source_id=source_id,
                record_url=record_url,
                jobs=utils.get_pydantic_model(
                    jobs, OptionalNullable[List[models.EmployeeJobInput]]
                ),
                compensations=utils.get_pydantic_model(
                    compensations,
                    OptionalNullable[List[models.EmployeeCompensationInput]],
                ),
                works_remote=works_remote,
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount]]
                ),
                tax_code=tax_code,
                tax_id=tax_id,
                dietary_preference=dietary_preference,
                food_allergies=food_allergies,
                probation_period=utils.get_pydantic_model(
                    probation_period, Optional[models.ProbationPeriod]
                ),
                tags=tags,
                row_version=row_version,
                deleted=deleted,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.employee, False, False, "json", models.EmployeeInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesUpdateResponse(
                update_employee_response=unmarshal_json_response(
                    Optional[models.UpdateEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        id_param: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        id: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        middle_name: OptionalNullable[str] = UNSET,
        display_name: OptionalNullable[str] = UNSET,
        preferred_name: OptionalNullable[str] = UNSET,
        initials: OptionalNullable[str] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        marital_status: OptionalNullable[str] = UNSET,
        partner: Optional[
            Union[models.PersonInput, models.PersonInputTypedDict]
        ] = None,
        division: OptionalNullable[str] = UNSET,
        division_id: OptionalNullable[str] = UNSET,
        department: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        department_name: OptionalNullable[str] = UNSET,
        team: OptionalNullable[Union[models.Team, models.TeamTypedDict]] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        company_name: OptionalNullable[str] = UNSET,
        employment_start_date: OptionalNullable[str] = UNSET,
        employment_end_date: OptionalNullable[str] = UNSET,
        leaving_reason: OptionalNullable[models.LeavingReason] = UNSET,
        employee_number: OptionalNullable[str] = UNSET,
        employment_status: OptionalNullable[models.EmploymentStatus] = UNSET,
        employment_role: Optional[
            Union[models.EmploymentRole, models.EmploymentRoleTypedDict]
        ] = None,
        ethnicity: OptionalNullable[str] = UNSET,
        manager: Optional[Union[models.Manager, models.ManagerTypedDict]] = None,
        direct_reports: OptionalNullable[List[str]] = UNSET,
        social_security_number: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        deceased_on: OptionalNullable[date] = UNSET,
        country_of_birth: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        gender: OptionalNullable[models.Gender] = UNSET,
        pronouns: OptionalNullable[str] = UNSET,
        preferred_language: OptionalNullable[str] = UNSET,
        languages: Optional[List[Nullable[str]]] = None,
        nationalities: Optional[List[Nullable[str]]] = None,
        photo_url: OptionalNullable[str] = UNSET,
        timezone: OptionalNullable[str] = UNSET,
        source: OptionalNullable[str] = UNSET,
        source_id: OptionalNullable[str] = UNSET,
        record_url: OptionalNullable[str] = UNSET,
        jobs: OptionalNullable[
            Union[List[models.EmployeeJobInput], List[models.EmployeeJobInputTypedDict]]
        ] = UNSET,
        compensations: OptionalNullable[
            Union[
                List[models.EmployeeCompensationInput],
                List[models.EmployeeCompensationInputTypedDict],
            ]
        ] = UNSET,
        works_remote: OptionalNullable[bool] = UNSET,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        bank_accounts: Optional[
            Union[List[models.BankAccount], List[models.BankAccountTypedDict]]
        ] = None,
        tax_code: OptionalNullable[str] = UNSET,
        tax_id: OptionalNullable[str] = UNSET,
        dietary_preference: OptionalNullable[str] = UNSET,
        food_allergies: OptionalNullable[List[str]] = UNSET,
        probation_period: Optional[
            Union[models.ProbationPeriod, models.ProbationPeriodTypedDict]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        row_version: OptionalNullable[str] = UNSET,
        deleted: OptionalNullable[bool] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesUpdateResponse:
        r"""Update Employee

        Update Employee

        :param id_param: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param id: A unique identifier for an object.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param middle_name: Middle name of the person.
        :param display_name: The name used to display the employee, often a combination of their first and last names.
        :param preferred_name: The name the employee prefers to be addressed by, which may be different from their legal name.
        :param initials: The initials of the person, usually derived from their first, middle, and last names.
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param title: The job title of the person.
        :param marital_status: The marital status of the employee.
        :param partner:
        :param division: The division the person is currently in. Usually a collection of departments or teams or regions.
        :param division_id: Unique identifier of the division this employee belongs to.
        :param department: The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field.
        :param department_id: Unique identifier of the department ID this employee belongs to.
        :param department_name: Name of the department this employee belongs to.
        :param team: The team the person is currently in.
        :param company_id: The unique identifier of the company.
        :param company_name: The name of the company.
        :param employment_start_date: A Start Date is the date that the employee started working at the company
        :param employment_end_date: An End Date is the date that the employee ended working at the company
        :param leaving_reason: The reason because the employment ended.
        :param employee_number: An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company.
        :param employment_status: The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status.
        :param employment_role:
        :param ethnicity: The ethnicity of the employee
        :param manager:
        :param direct_reports: Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee.
        :param social_security_number: A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions.
        :param birthday: The date of birth of the person.
        :param deceased_on: The date the person deceased.
        :param country_of_birth: Country code according to ISO 3166-1 alpha-2.
        :param description: A description of the object.
        :param gender: The gender represents the gender identity of a person.
        :param pronouns: The preferred pronouns of the person.
        :param preferred_language: language code according to ISO 639-1. For the United States - EN
        :param languages:
        :param nationalities:
        :param photo_url: The URL of the photo of a person.
        :param timezone: The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London.
        :param source: When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from.
        :param source_id: Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS).
        :param record_url:
        :param jobs:
        :param compensations:
        :param works_remote: Indicates if the employee works from a remote location.
        :param addresses:
        :param phone_numbers:
        :param emails:
        :param custom_fields:
        :param social_links:
        :param bank_accounts:
        :param tax_code:
        :param tax_id:
        :param dietary_preference: Indicate the employee's dietary preference.
        :param food_allergies: Indicate the employee's food allergies.
        :param probation_period:
        :param tags:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param deleted: Flag to indicate if the object is deleted.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesUpdateRequest(
            id_param=id_param,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            employee=models.EmployeeInput(
                id=id,
                first_name=first_name,
                last_name=last_name,
                middle_name=middle_name,
                display_name=display_name,
                preferred_name=preferred_name,
                initials=initials,
                salutation=salutation,
                title=title,
                marital_status=marital_status,
                partner=utils.get_pydantic_model(partner, Optional[models.PersonInput]),
                division=division,
                division_id=division_id,
                department=department,
                department_id=department_id,
                department_name=department_name,
                team=utils.get_pydantic_model(team, OptionalNullable[models.Team]),
                company_id=company_id,
                company_name=company_name,
                employment_start_date=employment_start_date,
                employment_end_date=employment_end_date,
                leaving_reason=leaving_reason,
                employee_number=employee_number,
                employment_status=employment_status,
                employment_role=utils.get_pydantic_model(
                    employment_role, Optional[models.EmploymentRole]
                ),
                ethnicity=ethnicity,
                manager=utils.get_pydantic_model(manager, Optional[models.Manager]),
                direct_reports=direct_reports,
                social_security_number=social_security_number,
                birthday=birthday,
                deceased_on=deceased_on,
                country_of_birth=country_of_birth,
                description=description,
                gender=gender,
                pronouns=pronouns,
                preferred_language=preferred_language,
                languages=languages,
                nationalities=nationalities,
                photo_url=photo_url,
                timezone=timezone,
                source=source,
                source_id=source_id,
                record_url=record_url,
                jobs=utils.get_pydantic_model(
                    jobs, OptionalNullable[List[models.EmployeeJobInput]]
                ),
                compensations=utils.get_pydantic_model(
                    compensations,
                    OptionalNullable[List[models.EmployeeCompensationInput]],
                ),
                works_remote=works_remote,
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount]]
                ),
                tax_code=tax_code,
                tax_id=tax_id,
                dietary_preference=dietary_preference,
                food_allergies=food_allergies,
                probation_period=utils.get_pydantic_model(
                    probation_period, Optional[models.ProbationPeriod]
                ),
                tags=tags,
                row_version=row_version,
                deleted=deleted,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.employee, False, False, "json", models.EmployeeInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesUpdateResponse(
                update_employee_response=unmarshal_json_response(
                    Optional[models.UpdateEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesDeleteResponse:
        r"""Delete Employee

        Delete Employee

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="DELETE",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesDeleteResponse(
                delete_employee_response=unmarshal_json_response(
                    Optional[models.DeleteEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.HrisEmployeesDeleteResponse:
        r"""Delete Employee

        Delete Employee

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.HrisEmployeesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/hris/employees/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.HrisEmployeesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="hris.employeesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.HrisEmployeesDeleteResponse(
                delete_employee_response=unmarshal_json_response(
                    Optional[models.DeleteEmployeeResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.HrisEmployeesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)
